/**
 * indexedDb plugin for jsonStoreDB
 * @param  $ jQuery namespace
 */
(function($){
	var iDB = window.indexedDB || window.webkitIndexedDB || window.mozIndexedDB || window.msIndexedDB;
	var iDBTransaction = window.IDBTransaction || window.webkitIDBTransaction || window.mozIDBTransaction || window.msIDBTransaction;
	var iDBKeyRange = window.IDBKeyRange || window.webkitIDBKeyRange || window.mozIDBKeyRange || window.msIDBKeyRange;

	var promise = {
		"db" : function(dbName, storeName) {			
			return $.Deferred(function(dfd){
				
				var request = iDB.open(dbName);
				var db;

				request.onblocked = function(e){
					console.log('request is blocked');
					dfd.reject(e);
				}

				request.onerror = function(e) {
					console.log(e);
					dfd.reject(e);
				}

				request.onsuccess = function(e) {
					//console.log('dbopen was a success');
					db = e.target.result;
					if(storeName && !db.objectStoreNames.contains(storeName)){
						var nv = parseInt(db.version || 0, 10) + 1;
						
						if(db.setVersion){
							var vRequest = db.setVersion(nv);
							vRequest.onsuccess = function() {
								db.createObjectStore(storeName, {keyPath: "key"});
								dfd.resolve(db, dbName);
							}
						}
						else {
							db.close();
							var vRequest = iDB.open(dbName, nv);
							vRequest.onupgradeneeded = function(e){
								e.target.result.createObjectStore(storeName, {keyPath: "key"});
							}
							vRequest.onsuccess = function(e) {
								dfd.resolve(e.target.result, dbName);
							}
							vRequest.onerror = function(e) {
								console.log(e);
								dfd.reject(e);
							}
							vRequest.onblocked = function(e) {
								console.log(e);
								dfd.reject(e);
							}
						}
					}
					else {
						dfd.resolve(db, dbName);
					}
				}
			}).promise();
		},

		"deleteDb" : function(dbName) {
			return $.Deferred(function(dfd) {
				var request = iDB.deleteDatabase(dbName);
				request.onsuccess = function(e) {
					dfd.resolve();
				}
				request.onerror = function(e) {
					dfd.reject();
				}
			}).promise();
		},

		"getDatabaseNames" : function() {
			return $.Deferred(function(dfd) {
				if(iDB.getDatabaseNames) {
					var request = iDB.getDatabaseNames();
					request.onsuccess = function(e) {
						var list = e.target.result, l = list.length, i = 0, tmp = [];
						for (;i < l;){
							tmp.push(list.item(i++));
						}
						dfd.resolve(tmp);
					}
					request.onerror = function(e) {

					}
				}else{
					dfd.reject('getDatabaseNames not supported!');
				}
			}).promise()
		},

		"transaction" : function(dbPromise, storeName, transactionType) {
			return $.Deferred(function(dfd) {
				dbPromise.then(function(db, dbName){
					try{
						var transaction = db.transaction(storeName, transactionType);
						dfd.resolve(transaction);
					}
					catch (e) {
						console.log(e);
						dfd.reject(e, db);
						return;
					}
					
				})

			}).promise();
		},

		"deleteObjectStore" : function(dbName, storeName) {
			var dbPromise = promise.db(dbName);

			if(!$.isArray(storeName)){
				storeName = [storeName];
			}

			return $.Deferred(function(dfd) {
				dbPromise.then(function(db) {
					var vRequest, nvRequired = false, i = 0;

					while(!nvRequired) {
						if(db.objectStoreNames.contains(storeName[i])){
							nvRequired = true;
						}
						if(!storeName[i++]){
							break;
						}
					}
					
					if(nvRequired) {
						if(db.setVersion){
							vRequest = db.setVersion(parseInt(db.version || 0, 10) + 1);
							vRequest.onsuccess = function() {
								try {
									$.each(storeName, function(idx, name){
										db.deleteObjectStore(name);
									});
									
									dfd.resolve();
								}
								catch(e){
									dfd.reject(e);
								}
							}
							vRequest.onerror = function(e) {
								dfd.reject(e);
							}
						}
						else {
							var nv = parseInt(db.version || 0, 10) + 1;
							db.close();
							vRequest = iDB.open(dbName, nv);
							vRequest.onupgradeneeded = function(e) {
								$.each(storeName, function(idx, name){
									e.target.result.deleteObjectStore(name);
								});
							}
							vRequest.onsuccess = function(e) {
								dfd.resolve();
							}
							vRequest.onerror = function(e) {
								dfd.reject();
							}
						}
					}
					else {
						dfd.reject('no objectStore with name: ' + storeName);
					}
				})
			}).promise();
		},

		"getObjectStoreNames" : function(dbName) {
			var dbPromise = promise.db(dbName);

			return $.Deferred(function(dfd) {
				dbPromise.then(function(db) {
					var list = db.objectStoreNames, l = list.length, i = 0, tmp = [];

					for (;i < l;){
						tmp.push(list.item(i++));
					}

					dfd.resolve(tmp);
				})
			}).promise();
		},

		"objectStore" : function(dbName, storeName) {
			var dbPromise = promise.db(dbName, storeName);

			function getObjectStorePromise(type){
				return $.Deferred(function(dfd) {
					var TRANSACTION_TYPE = type == 'READ_WRITE' ? iDBTransaction.READ_WRITE : undefined;
					promise.transaction(dbPromise, storeName, TRANSACTION_TYPE).then(function(transaction){
						var store = transaction.objectStore(storeName);
						dfd.resolve(store);
					})
				}).promise();
			}



			var crudOp = function(op, args, transactionType){
				var objectStorePromise = getObjectStorePromise(transactionType);

				return $.Deferred(function(dfd) {
					objectStorePromise.then(function(objectStore, transaction) {
						var request, cursor, kRange, tmp = [];

						switch(op) {
							case 'add' :
								var data = args[0];
								if(!data.key){
									data.key = args[1];
								}

								request = objectStore.add(data);
								request.onsuccess = function(e) {
									console.log('it was added');
									dfd.resolve(data);
								}
								request.onerror = function(e) {
									dfd.reject(e);
								}
								break;

							case 'update' :
								var data = args[0];
								
								kRange = iDBKeyRange.only(data.key);
								
								cursor = objectStore.openCursor(kRange);
								cursor.onsuccess = function(e) {
									var c = e.target.result;
									if(c) {
										c.update(data);
										dfd.resolve(data);
									}
								}
								cursor.onerror = function(e) {
									console.log(e);
									console.log('it was a fail');
									dfd.reject(e);
								}
								break;
								

							case 'get' :
								request = objectStore.get(args[0]);
								request.onsuccess = function(e) {
									dfd.resolve(e.target.result);
								}
								request.onerror = function(e) {
									dfd.reject(e);
								}
								break;

							case 'getKeys' : 
								cursor = objectStore.openCursor();
								cursor.onsuccess = function(e) {
									var c = e.target.result;
									if(c) {
										tmp.push(c.key);
										c.continue();
									}
									else {
										dfd.resolve(tmp);
									}
								}
								cursor.onerror = function(e) {
									dfd.reject(e);
								}
								break;

							case 'getAll' :
								cursor = objectStore.openCursor();
								cursor.onsuccess = function(e) {
									var c = e.target.result;
									if(c) {
										tmp.push(c.value);
										c.continue();
									}
									else {
										dfd.resolve(tmp);
									}
								}
								cursor.onerror = function(e) {
									dfd.reject(e);
								}
								break;

							case 'delete' :
								request = objectStore.delete(args[0]);
								request.onsuccess = function(e) {
									dfd.resolve();
								} 
								request.onerror = function(e) {
									dfd.reject(e);
								}
								break;
						}
					});
				}).promise();
			}

			var store = {};

			$.extend(store, {
				
				"add" : function(data, key) {
					return crudOp('save', [data, key], "READ_WRITE");
				},

				"update" : function(data) {
					return crudOp('update', [data], "READ_WRITE");
				},

				"get" : function(key) {
					return crudOp('get', [key]);
				},

				"getKeys" : function() {
					return crudOp('getKeys');
				},

				"getAll" : function() {
					return crudOp('getAll');
				},

				"delete" : function(key) {
					return crudOp('delete', [key], "READ_WRITE");
				}
			});

			return store;
		}
	}

	$.extend($.jsonStoreDB_plugins.indexedDB,{

		"getDeleteDbPromise" : function(dbName) {
			return promise.deleteDb(dbName);
		},

		"getObjectStorePromise" : function(dbName, storeName) {
			return promise.objectStore(dbName, storeName);
		},

		"getDeleteObjectStorePromise" : function(dbName, storeName) {
			return promise.deleteObjectStore(dbName, storeName);
		},

		"getObjectStoreNamesPromise" : function(dbName) {
			return promise.getObjectStoreNames(dbName);
		},

		"getDatabaseNamesPromise" : function() {
			return promise.getDatabaseNames();
		}
	})
})(jQuery);